import { Queue } from "../../common/queue";
import { TreeChanges } from "../../transition/interface";
import { Transition } from "../../transition/transition";
import { StateDeclaration } from "../interface";
import { StateService } from "../stateService";
import { UrlRouter } from "../../url/urlRouter";
/**
 * This class:
 *
 * * Takes a blank transition object and adds all the hooks necessary for it to behave like a state transition.
 *
 * * Runs the transition, returning a chained promise which:
 *   * transforms the resolved Transition.promise to the final destination state.
 *   * manages the rejected Transition.promise, checking for Dynamic or Redirected transitions
 *
 * * Registers a handler to update global $state data such as "active transitions" and "current state/params"
 *
 * * Registers view hooks, which maintain the list of active view configs and sync with/update the ui-views
 *
 * * Registers onEnter/onRetain/onExit hooks which delegate to the state's hooks of the same name, at the appropriate time
 *
 * * Registers eager and lazy resolve hooks
 */
export declare class TransitionManager {
    private transition;
    private $transitions;
    private $urlRouter;
    private $view;
    private $state;
    private $stateParams;
    private $q;
    private activeTransQ;
    private changeHistory;
    private treeChanges;
    private enterExitHooks;
    private viewHooks;
    private resolveHooks;
    constructor(transition: Transition, $transitions: any, $urlRouter: UrlRouter, $view: any, $state: StateService, $stateParams: any, $q: any, activeTransQ: Queue<Transition>, changeHistory: Queue<TreeChanges>);
    runTransition(): Promise<any>;
    registerUpdateGlobalState(): void;
    updateGlobalState(): void;
    transRejected(error: any): (StateDeclaration | Promise<any>);
    updateStateParams(): void;
    private _redirectMgr(redirect);
}
